/**
 * Copyright (c) 2014 - 2018, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup blinky_example_main main.c
 * @{
 * @ingroup blinky_example
 * @brief Blinky Example Application main file.
 *
 * This file contains the source code for a sample application to blink LEDs.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include "nrf_delay.h"
#include "boards.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "SEGGER_RTT.h"
#include "SEGGER_RTT_Conf.h"

__attribute__((at(0x20009000))) unsigned int gRetained;
// An unsigned int in an uninitialised RAM area
#if defined(__CC_ARM) || (defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050))
//__attribute__ ((section(".bss.uninitialised"),zero_init))

    uint32_t *ptr = &gRetained;
#elif defined(__GNUC__)
__attribute__ ((section(".uninitialised")))
unsigned int gRetained;
#elif defined(__ICCARM__)
unsigned int gRetained @ ".uninitialised";
#endif


// Watchdog interrupt handler
void WDT_IRQHandler(void)
{
    gRetained++;
    NRF_WDT->EVENTS_TIMEOUT = 0;
    SEGGER_RTT_printf(0, "WDT_IRQHandler incrementing RAM variable to %d.\n", gRetained);
}

static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}

void lfclk_start()
{
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
    NRF_CLOCK->LFCLKSRC = (CLOCK_LFCLKSRC_SRC_Xtal << CLOCK_LFCLKSRC_SRC_Pos);
    __DSB();     //<-- This will wait until write buffers are emptied.
    NRF_CLOCK->TASKS_LFCLKSTART = 1;
    while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0){  }
}
    
// Entry point
int main()
{

        log_init();
    SEGGER_RTT_printf(0, "\nStarting up and setting the watchdog timer to 5 seconds.\n");
    // Setting watchdog to 10 seconds
    // Set timeout value timeout [s] = ( CRV + 1 ) / 32768
    lfclk_start();
    NRF_WDT->CRV = (5 * 32768) - 1;
    NVIC_SetPriority(WDT_IRQn, 7);
    NVIC_ClearPendingIRQ(WDT_IRQn);
    NVIC_EnableIRQ(WDT_IRQn);
    NRF_WDT->INTENSET = 1;
    NRF_WDT->TASKS_START = 1;

    SEGGER_RTT_printf(0, "Retained RAM variable is %d.\n", gRetained);
    if (gRetained > 2) {
        SEGGER_RTT_printf(0, "Setting retained RAM variable to 0 and resetting...\n");
        gRetained = 0;
        nrf_delay_ms(1000);
        //NVIC_SystemReset();
    }

    gRetained++;
    SEGGER_RTT_printf(0, "Retained RAM variable incremented to %d.\n", gRetained);

    if (gRetained < 2) {
        SEGGER_RTT_printf(0, "Resetting...\n");
        nrf_delay_ms(10000);
        //NVIC_SystemReset();
    } else {
        SEGGER_RTT_printf(0, "Now waiting for 10 seconds so that the watchdog goes off, which should increment the retained RAM variable to %d.\n",
        gRetained + 1);
        nrf_delay_ms(10000);
    }
 
    SEGGER_RTT_printf(0, "Should never get here.\n");
    while(1) {}
}

/**
 *@}
 **/
